"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../../../src/client");
const clientIxParamBuilder_1 = require("../../../src/clientIxParamBuilder");
const constants_1 = require("../../../src/constants");
const DEVNET_SERUM3_MARKETS = new Map([
    ['SOL/USDC', '6xYbSQyhajUqyatJDdkonpj7v41bKeEBWpf7kwRh5X7A'],
]);
const DEVNET_MINTS = new Map([
    ['USDC', '8FRFC6MoGGkMFQwngccyu69VnYbzykGeez7ignHVAFSN'],
    ['USDT', 'DAwBSXe6w9g37wdE2tCrFbho3QHKZi4PjuBytQCULap2'],
    ['SOL', 'So11111111111111111111111111111111111111112'],
]);
const DEVNET_ORACLES = new Map([
    ['SOL', 'J83w4HKfqxwcq3BEMMkPFSppX3gqekLyLJBexebFVkix'],
]);
const GROUP_NUM = 2814;
async function main() {
    let sig;
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection('https://mango.devnet.rpcpool.com', options);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.ADMIN_KEYPAIR, 'utf-8'))));
    const adminWallet = new anchor_1.Wallet(admin);
    console.log(`Admin ${adminWallet.publicKey.toBase58()}`);
    const adminProvider = new anchor_1.AnchorProvider(connection, adminWallet, options);
    const client = await client_1.MangoClient.connect(adminProvider, 'devnet', constants_1.MANGO_V4_ID['devnet'], {
        idsSource: 'get-program-accounts',
    });
    // group
    console.log(`Creating Group...`);
    const insuranceMint = new web3_js_1.PublicKey(DEVNET_MINTS.get('USDC'));
    try {
        await client.groupCreate(GROUP_NUM, true, 0, insuranceMint);
    }
    catch (error) {
        console.log(error);
    }
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(`...registered group ${group.publicKey}`);
    // stub usdc oracle + register token 0
    console.log(`Registering USDC...`);
    const usdcDevnetMint = new web3_js_1.PublicKey(DEVNET_MINTS.get('USDC'));
    try {
        sig = await client.stubOracleCreate(group, insuranceMint, 1.0);
        const usdcDevnetOracle = (await client.getStubOracle(group, insuranceMint))[0];
        console.log(`...registered stub oracle ${usdcDevnetOracle}, https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        sig = await client.tokenRegister(group, usdcDevnetMint, usdcDevnetOracle.publicKey, web3_js_1.PublicKey.default, 0, // tokenIndex
        'USDC', {
            ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
            oracleConfig: {
                confFilter: 10000,
                maxStalenessSlots: null,
            },
        });
        await group.reloadAll(client);
        const bank = group.getFirstBankByMint(usdcDevnetMint);
        console.log(`...registered token bank ${bank.publicKey}, https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        await group.reloadAll(client);
        // eslint-disable-next-line
    }
    catch (error) { }
    // register token 4
    console.log(`Registering SOL...`);
    const solDevnetMint = new web3_js_1.PublicKey(DEVNET_MINTS.get('SOL'));
    const solDevnetOracle = new web3_js_1.PublicKey(DEVNET_ORACLES.get('SOL'));
    try {
        sig = await client.tokenRegister(group, solDevnetMint, solDevnetOracle, web3_js_1.PublicKey.default, 4, // tokenIndex
        'SOL', {
            ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
            oracleConfig: {
                confFilter: 10000,
                maxStalenessSlots: null,
            },
        });
        await group.reloadAll(client);
        const bank = group.getFirstBankByMint(solDevnetMint);
        console.log(`...registered token bank ${bank.publicKey}, https://explorer.solana.com/tx/${sig}?cluster=devnet`);
    }
    catch (error) {
        console.log(error);
    }
    console.log(`Registering USDT...`);
    const usdtDevnetMint = new web3_js_1.PublicKey(DEVNET_MINTS.get('USDT'));
    const usdcDevnetOracle = (await client.getStubOracle(group, insuranceMint))[0];
    try {
        sig = await client.tokenRegister(group, usdtDevnetMint, usdcDevnetOracle.publicKey, web3_js_1.PublicKey.default, 5, // tokenIndex
        'USDT', {
            ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
            oracleConfig: {
                confFilter: 10000,
                maxStalenessSlots: null,
            },
        });
        await group.reloadAll(client);
        const bank = group.getFirstBankByMint(solDevnetMint);
        console.log(`...registered token bank ${bank.publicKey}, https://explorer.solana.com/tx/${sig}?cluster=devnet`);
    }
    catch (error) {
        console.log(error);
    }
    // register serum market
    console.log(`Registering serum3 markets...`);
    const serumMarketExternalPk = new web3_js_1.PublicKey(DEVNET_SERUM3_MARKETS.get('SOL/USDC'));
    try {
        sig = await client.serum3RegisterMarket(group, serumMarketExternalPk, group.getFirstBankByMint(solDevnetMint), group.getFirstBankByMint(insuranceMint), 0, 'SOL/USDC', 0);
        await group.reloadAll(client);
        const serum3Market = group.getSerum3MarketByExternalMarket(serumMarketExternalPk);
        console.log(`...registered serum market ${serum3Market.publicKey}, https://explorer.solana.com/tx/${sig}?cluster=devnet`);
    }
    catch (error) {
        console.log(error);
    }
    // register perp market
    let count = 0;
    console.log(`Registering perp market...`);
    for (const market of ['SOL-PERP1', 'SOL-PERP2', 'SOL-PERP3']) {
        count = count + 1;
        try {
            sig = await client.perpCreateMarket(group, new web3_js_1.PublicKey(DEVNET_ORACLES.get('SOL')), count, market, { confFilter: 10000, maxStalenessSlots: null }, 6, 10, 100, 0.975, 0.95, 1.025, 1.05, 0.95, 0.9, 0.012, 0.0002, 0.0, 0, 0.05, 0.05, 100, true, 1000, 1000000, 0.05, 0, 1.0, 2 * 60 * 60, 0.025, 0);
            await group.reloadAll(client);
            const perpMarket = group.getPerpMarketByMarketIndex(count);
            console.log(`...registered perp market ${perpMarket.publicKey}, https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
        }
    }
    process.exit();
}
main();
