"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../../src/client");
const clientIxParamBuilder_1 = require("../../src/clientIxParamBuilder");
const constants_1 = require("../../src/constants");
const TESTNET_MINTS = new Map([
    ['USDC', 'AkdEhBMvaDD1UbGMD3Hxnr3h5PEL2R8PaCDAssCN28WV'],
]);
// TODO: should these constants be baked right into client.ts or even program?
const NET_BORROWS_LIMIT_NATIVE = 1 * Math.pow(10, 7) * Math.pow(10, 6);
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
async function main() {
    let sig;
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection('https://testnet.dev2.eclipsenetwork.xyz', options);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.ADMIN_KEYPAIR, 'utf-8'))));
    const adminWallet = new anchor_1.Wallet(admin);
    console.log(`Admin ${adminWallet.publicKey.toBase58()}`);
    const adminProvider = new anchor_1.AnchorProvider(connection, adminWallet, options);
    const client = await client_1.MangoClient.connect(adminProvider, 'testnet', constants_1.MANGO_V4_ID['testnet'], {
        idsSource: 'get-program-accounts',
    });
    // group
    console.log(`Creating Group...`);
    const insuranceMint = new web3_js_1.PublicKey(TESTNET_MINTS.get('USDC'));
    try {
        await client.groupCreate(GROUP_NUM, true, 0, insuranceMint);
    }
    catch (error) {
        console.log(error);
    }
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(`...registered group ${group.publicKey}`);
    // stub usdc oracle + register token 0
    console.log(`Registering USDC...`);
    const usdcDevnetMint = new web3_js_1.PublicKey(TESTNET_MINTS.get('USDC'));
    sig = await client.stubOracleCreate(group, usdcDevnetMint, 1.0);
    const usdcDevnetOracle = (await client.getStubOracle(group, usdcDevnetMint))[0];
    console.log(`...registered stub oracle ${usdcDevnetOracle}, https://explorer.dev.eclipsenetwork.xyz/tx/${sig.signature}?cluster=testnet`);
    sig = await client.tokenRegister(group, usdcDevnetMint, usdcDevnetOracle.publicKey, web3_js_1.PublicKey.default, 0, // tokenIndex
    'USDC', {
        ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
        initAssetWeight: 1,
        maintAssetWeight: 1,
        initLiabWeight: 1,
        maintLiabWeight: 1,
        liquidationFee: 0,
        netBorrowLimitPerWindowQuote: NET_BORROWS_LIMIT_NATIVE,
    });
    await group.reloadAll(client);
    const bank = group.getFirstBankByMint(usdcDevnetMint);
    console.log(`...registered token bank ${bank.publicKey}, https://explorer.dev.eclipsenetwork.xyz/tx/${sig.signature}?cluster=testnet`);
    await group.reloadAll(client);
    process.exit();
}
main();
