"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../../src/client");
const constants_1 = require("../../src/constants");
const TESTNET_MINTS = new Map([
    ['USDC', 'AkdEhBMvaDD1UbGMD3Hxnr3h5PEL2R8PaCDAssCN28WV'],
]);
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection('https://testnet.dev2.eclipsenetwork.xyz', options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.USER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, 'testnet', constants_1.MANGO_V4_ID['testnet'], {
        idsSource: 'get-program-accounts',
    });
    console.log(`User ${userWallet.publicKey.toBase58()}`);
    // fetch group
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.ADMIN_KEYPAIR, 'utf-8'))));
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(`Group ${group.publicKey}`);
    // create + fetch account
    console.log(`Creating mangoaccount...`);
    let mangoAccount = (await client.getMangoAccountForOwner(group, user.publicKey, 0));
    if (!mangoAccount) {
        await client.createMangoAccount(group, 0);
        mangoAccount = (await client.getMangoAccountForOwner(group, user.publicKey, 0));
    }
    console.log(`...created/found mangoAccount ${mangoAccount.publicKey}`);
    // deposit USDC
    const sig = await client.tokenDeposit(group, mangoAccount, new web3_js_1.PublicKey(TESTNET_MINTS.get('USDC')), 50);
    console.log(`deposited token, https://explorer.dev.eclipsenetwork.xyz/tx/${sig.signature}?cluster=testnet`);
    process.exit();
}
main();
