"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const serum3_1 = require("../../src/accounts/serum3");
const builder_1 = require("../../src/builder");
const client_1 = require("../../src/client");
const clientIxParamBuilder_1 = require("../../src/clientIxParamBuilder");
const constants_1 = require("../../src/constants");
const utils_1 = require("../../src/utils");
const spl_1 = require("../../src/utils/spl");
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
const MAINNET_MINTS = new Map([
    ['USDC', 'EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v'], // 0
    ['USDT', 'Es9vMFrzaCERmJfrF4H2FYD4KCoNkY11McCe8BenwNYB'], // 1
    ['DAI', 'EjmyN6qEC1Tf1JxiG1ae7UTJhUxSwk1TCWNWqxWV4J6o'], // 2
    ['ETH', '7vfCXTUXx5WJV5JADk17DUJ4ksgau7utNKj4b963voxs'], // 3 Ether (Portal)
    ['SOL', 'So11111111111111111111111111111111111111112'], // 4 Wrapped SOL
    ['MSOL', 'mSoLzYCxHdYgdzU16g5QSh3i5K3z3KZK7ytfqcJm7So'], // 5
    ['MNGO', 'MangoCzJ36AjZyKwVj3VnYU4GTonjfVEnJmvvWaxLac'], // 6
    ['BONK', 'DezXAZ8z7PnrnRJjz3wXBoRgixCa6xjnB7YaB1pPB263'], // 7
]);
const MAINNET_ORACLES = new Map([
    // USDC - stub oracle
    ['USDT', '3vxLXJqLqF3JG5TCbYycbKWRBbCJQLxQmBGCkyqEEefL'],
    ['DAI', 'CtJ8EkqLmeYyGB8s4jevpeNsvmD4dxVR2krfsDLcvV8Y'],
    ['ETH', 'JBu1AL4obBcCMqKBBxhpWCNUt136ijcuMZLFvTP7iWdB'],
    ['SOL', 'H6ARHf6YXhGYeQfUzQNGk6rDNnLBQKrenN712K4AQJEG'],
    ['MSOL', 'E4v1BBgoso9s64TQvmyownAVJbhbEPGyzA3qn4n46qj9'],
    // ['MNGO', '79wm3jjcPr6RaNQ4DGvP5KxG1mNd3gEBsg6FsNVFezK4'], // pyth
    ['MNGO', '5xUoyPG9PeowJvfai5jD985LiRvo58isaHrmmcBohi3Y'], // switchboard
    ['BTC', 'GVXRSBjFk6e6J3NbVPXohDJetcTjaeeuykUpbQF8UoMU'],
    ['BONK', '4SZ1qb4MtSUrZcoeaeQ3BDzVCyqxw3VwSFpPiMTmn4GE'],
]);
// External markets are matched with those in https://github.com/openbook-dex/openbook-ts/blob/master/packages/serum/src/markets.json
const MAINNET_SERUM3_MARKETS = new Map([
    ['SOL/USDC', '8BnEgHoWFysVcuFFX7QztDmzuH8r5ZFvyP3sYwn1XTh6'],
]);
const { MB_CLUSTER_URL, MB_PAYER_KEYPAIR, MB_PAYER3_KEYPAIR } = process.env;
const MIN_VAULT_TO_DEPOSITS_RATIO = 0.2;
const NET_BORROWS_WINDOW_SIZE_TS = 24 * 60 * 60;
const NET_BORROW_LIMIT_PER_WINDOW_QUOTE = (0, utils_1.toNative)(1000000, 6).toNumber();
const defaultOracleConfig = {
    confFilter: 0.1,
    maxStalenessSlots: null,
};
// hoping that dynamic rate parameter adjustment would be enough to tune their rates to the markets needs
const defaultInterestRate = {
    adjustmentFactor: 0.004, // rate parameters are chosen to be the same for all high asset weight tokens,
    util0: 0.7,
    rate0: 0.1,
    util1: 0.85,
    rate1: 0.2,
    maxRate: 2.0,
};
async function buildAdminClient() {
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(MB_PAYER3_KEYPAIR, 'utf-8'))));
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(MB_CLUSTER_URL, options);
    const adminWallet = new anchor_1.Wallet(admin);
    console.log(`Admin ${adminWallet.publicKey.toBase58()}`);
    const adminProvider = new anchor_1.AnchorProvider(connection, adminWallet, options);
    const client = await client_1.MangoClient.connect(adminProvider, 'mainnet-beta', constants_1.MANGO_V4_ID['mainnet-beta'], {
        idsSource: 'get-program-accounts',
    });
    const creator = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(MB_PAYER_KEYPAIR, 'utf-8'))));
    return [client, admin, creator];
}
async function buildUserClient(userKeypair) {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(MB_CLUSTER_URL, options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(userKeypair, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, 'mainnet-beta', constants_1.MANGO_V4_ID['mainnet-beta']);
    const creator = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(MB_PAYER_KEYPAIR, 'utf-8'))));
    console.log(`Creator ${creator.publicKey.toBase58()}`);
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    return [client, group, user];
}
async function createGroup() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    console.log(`Creating Group...`);
    const insuranceMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('USDC'));
    await client.groupCreate(GROUP_NUM, true, 2, insuranceMint);
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(`...registered group ${group.publicKey}`);
}
async function changeAdmin() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    console.log(`Changing admin...`);
    await client.groupEdit(group, new web3_js_1.PublicKey('DSiGNQaKhFCSZbg4HczqCtPAPb1xV51c9GfbfqcVKTB4'), new web3_js_1.PublicKey('DSiGNQaKhFCSZbg4HczqCtPAPb1xV51c9GfbfqcVKTB4'), new web3_js_1.PublicKey('DSiGNQaKhFCSZbg4HczqCtPAPb1xV51c9GfbfqcVKTB4'));
}
async function setDepositLimit() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    console.log(`Setting a deposit limit...`);
    await client.groupEdit(group, new web3_js_1.PublicKey('DSiGNQaKhFCSZbg4HczqCtPAPb1xV51c9GfbfqcVKTB4'), new web3_js_1.PublicKey('DSiGNQaKhFCSZbg4HczqCtPAPb1xV51c9GfbfqcVKTB4'), new web3_js_1.PublicKey('DSiGNQaKhFCSZbg4HczqCtPAPb1xV51c9GfbfqcVKTB4'), undefined, undefined, (0, utils_1.toNative)(200, 6));
}
async function registerTokens() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    console.log(`Creating USDC stub oracle...`);
    const usdcMainnetMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('USDC'));
    await client.stubOracleCreate(group, usdcMainnetMint, 1.0);
    const usdcMainnetOracle = (await client.getStubOracle(group, usdcMainnetMint))[0];
    console.log(`...created stub oracle ${usdcMainnetOracle.publicKey}`);
    console.log(`Registering USDC...`);
    await client.tokenRegister(group, usdcMainnetMint, usdcMainnetOracle.publicKey, web3_js_1.PublicKey.default, 0, 'USDC', {
        ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
        initAssetWeight: 1,
        maintAssetWeight: 1,
        initLiabWeight: 1,
        maintLiabWeight: 1,
        liquidationFee: 0,
        netBorrowLimitPerWindowQuote: NET_BORROW_LIMIT_PER_WINDOW_QUOTE,
    });
    console.log(`Registering USDT...`);
    const usdtMainnetMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('USDT'));
    const usdtMainnetOracle = new web3_js_1.PublicKey(MAINNET_ORACLES.get('USDT'));
    await client.tokenRegister(group, usdtMainnetMint, usdtMainnetOracle, web3_js_1.PublicKey.default, 1, 'USDT', {
        ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
        maintAssetWeight: 0.95,
        initAssetWeight: 0.9,
        maintLiabWeight: 1.05,
        initLiabWeight: 1.1,
        liquidationFee: 0.025,
        netBorrowLimitPerWindowQuote: NET_BORROW_LIMIT_PER_WINDOW_QUOTE,
    });
    console.log(`Registering DAI...`);
    const daiMainnetMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('DAI'));
    const daiMainnetOracle = new web3_js_1.PublicKey(MAINNET_ORACLES.get('DAI'));
    await client.tokenRegister(group, daiMainnetMint, daiMainnetOracle, web3_js_1.PublicKey.default, 2, 'DAI', {
        ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
        maintAssetWeight: 0.95,
        initAssetWeight: 0.9,
        maintLiabWeight: 1.05,
        initLiabWeight: 1.1,
        liquidationFee: 0.025,
        netBorrowLimitPerWindowQuote: NET_BORROW_LIMIT_PER_WINDOW_QUOTE,
    });
    console.log(`Registering ETH...`);
    const ethMainnetMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('ETH'));
    const ethMainnetOracle = new web3_js_1.PublicKey(MAINNET_ORACLES.get('ETH'));
    await client.tokenRegister(group, ethMainnetMint, ethMainnetOracle, web3_js_1.PublicKey.default, 3, 'ETH', {
        ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
        maintAssetWeight: 0.9,
        initAssetWeight: 0.8,
        maintLiabWeight: 1.1,
        initLiabWeight: 1.2,
        liquidationFee: 0.05,
        netBorrowLimitPerWindowQuote: NET_BORROW_LIMIT_PER_WINDOW_QUOTE,
    });
    console.log(`Registering SOL...`);
    const solMainnetMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('SOL'));
    const solMainnetOracle = new web3_js_1.PublicKey(MAINNET_ORACLES.get('SOL'));
    await client.tokenRegister(group, solMainnetMint, solMainnetOracle, web3_js_1.PublicKey.default, 4, 'SOL', {
        ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
        maintAssetWeight: 0.9,
        initAssetWeight: 0.8,
        maintLiabWeight: 1.1,
        initLiabWeight: 1.2,
        liquidationFee: 0.05,
        netBorrowLimitPerWindowQuote: NET_BORROW_LIMIT_PER_WINDOW_QUOTE,
    });
    console.log(`Registering MSOL...`);
    const msolMainnetMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('MSOL'));
    const msolMainnetOracle = new web3_js_1.PublicKey(MAINNET_ORACLES.get('MSOL'));
    await client.tokenRegister(group, msolMainnetMint, msolMainnetOracle, web3_js_1.PublicKey.default, 5, 'MSOL', {
        ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
        maintAssetWeight: 0.9,
        initAssetWeight: 0.8,
        maintLiabWeight: 1.1,
        initLiabWeight: 1.2,
        liquidationFee: 0.05,
        netBorrowLimitPerWindowQuote: NET_BORROW_LIMIT_PER_WINDOW_QUOTE,
    });
    console.log(`Registering MNGO...`);
    await client.groupEdit(group, group.admin, group.admin);
    const mngoMainnetMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('MNGO'));
    const mngoMainnetOracle = new web3_js_1.PublicKey(MAINNET_ORACLES.get('MNGO'));
    await client.tokenRegisterTrustless(group, mngoMainnetMint, mngoMainnetOracle, 6, 'MNGO');
    console.log(`Registering BONK...`);
    const bonkMainnetMint = new web3_js_1.PublicKey(MAINNET_MINTS.get('BONK'));
    const bonkMainnetOracle = new web3_js_1.PublicKey(MAINNET_ORACLES.get('BONK'));
    await client.tokenRegisterTrustless(group, bonkMainnetMint, bonkMainnetOracle, 7, 'BONK');
    // log tokens/banks
    await group.reloadAll(client);
    for (const bank of await Array.from(group.banksMapByMint.values())
        .flat()
        .sort((a, b) => a.tokenIndex - b.tokenIndex)) {
        console.log(`${bank.toString()}`);
    }
}
async function registerSerum3Markets() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    // Register SOL serum market
    await client.serum3RegisterMarket(group, new web3_js_1.PublicKey(MAINNET_SERUM3_MARKETS.get('SOL/USDC')), group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('SOL'))), group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('USDC'))), 0, 'SOL/USDC', 0.5);
}
async function createUser(userKeypair) {
    const result = await buildUserClient(userKeypair);
    const client = result[0];
    const group = result[1];
    const user = result[2];
    console.log(`Creating MangoAccount...`);
    const mangoAccount = await client.getMangoAccountForOwner(group, user.publicKey, 0);
    if (!mangoAccount) {
        throw new Error(`MangoAccount not found for user ${user.publicKey}`);
    }
    console.log(`...created MangoAccount ${mangoAccount.publicKey.toBase58()}`);
}
async function depositForUser(userKeypair) {
    const result = await buildUserClient(userKeypair);
    const client = result[0];
    const group = result[1];
    const user = result[2];
    const mangoAccount = await client.getMangoAccountForOwner(group, user.publicKey, 0);
    await client.tokenDeposit(group, mangoAccount, new web3_js_1.PublicKey(MAINNET_MINTS.get('USDC')), 10);
    await mangoAccount.reload(client);
    console.log(`...deposited 10 USDC`);
}
async function registerPerpMarkets() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    await client.perpCreateMarket(group, new web3_js_1.PublicKey(MAINNET_ORACLES.get('BTC')), 0, 'BTC-PERP', defaultOracleConfig, 6, 10, // 0.1$ is the min tick
    100, // if btc price is 20k, one base lot would be 2$
    0.975, 0.95, 1.025, 1.05, 0.95, 0.9, 0.0125, -0.0001, 0.0004, 5, // note: quote native
    -0.05, 0.05, 100, // if btc is at 20k, this is 200$
    true, 1000, // solana tx fee is currently 50 native quote at a sol price of 10$
    1000000, 0.01, // less than liquidationFee
    0, 1.0, 2 * 60 * 60, 0.025, 0);
    await client.perpCreateMarket(group, new web3_js_1.PublicKey(MAINNET_ORACLES.get('MNGO')), 1, 'MNGO-PERP-OLD', defaultOracleConfig, 6, 100, // 0.0001$ is the min tick
    1000000, // if mngo price is 1 cent, one base lot would be 1 cent
    0.995, 0.99, // 100x leverage
    1.005, 1.01, 0, 0, 0.0025, -0.0001, 0.0004, 5, -0.05, 0.05, 1000, // if mngo price 1 cent, this is 10$
    false, 1000, 1000000, 0.001, // less than liquidationFee
    0, 1.0, 2 * 60 * 60, 0.2, // 20% positive pnl liquidation fee?
    0);
}
async function changeTokenOracle() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    const bank = group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('MNGO')));
    await client.tokenEdit(group, bank.mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams)
        .oracle(new web3_js_1.PublicKey(MAINNET_ORACLES.get('MNGO')))
        .build());
}
async function makeTokenReduceonly() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    const bank = group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('DAI')));
    await client.tokenEdit(group, bank.mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams).reduceOnly(1).build());
}
async function changeMaxStalenessSlots() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    for (const bank of Array.from(group.banksMapByTokenIndex.values()).flat()) {
        await client.tokenEdit(group, bank.mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams)
            .oracleConfig({
            confFilter: 0.1,
            maxStalenessSlots: 120,
        })
            .build());
    }
    for (const perpMarket of Array.from(group.perpMarketsMapByMarketIndex.values())) {
        await client.perpEditMarket(group, perpMarket.perpMarketIndex, (0, builder_1.Builder)(clientIxParamBuilder_1.NullPerpEditParams)
            .oracleConfig({
            confFilter: 0.1,
            maxStalenessSlots: 120,
        })
            .build());
    }
}
async function changeStartQuote() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    await client.tokenEdit(group, group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('USDT'))).mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams)
        .depositWeightScaleStartQuote((0, utils_1.toNative)(1000000, 6).toNumber())
        .borrowWeightScaleStartQuote((0, utils_1.toNative)(1000000, 6).toNumber())
        .build());
    await client.tokenEdit(group, group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('ETH'))).mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams)
        .depositWeightScaleStartQuote((0, utils_1.toNative)(100000, 6).toNumber())
        .borrowWeightScaleStartQuote((0, utils_1.toNative)(100000, 6).toNumber())
        .build());
    await client.tokenEdit(group, group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('SOL'))).mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams)
        .depositWeightScaleStartQuote((0, utils_1.toNative)(5000000, 6).toNumber())
        .borrowWeightScaleStartQuote((0, utils_1.toNative)(5000000, 6).toNumber())
        .build());
    await client.tokenEdit(group, group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('MSOL'))).mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams)
        .depositWeightScaleStartQuote((0, utils_1.toNative)(1000000, 6).toNumber())
        .borrowWeightScaleStartQuote((0, utils_1.toNative)(1000000, 6).toNumber())
        .build());
    await client.tokenEdit(group, group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('MNGO'))).mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams)
        .depositWeightScaleStartQuote((0, utils_1.toNative)(5000, 6).toNumber())
        .borrowWeightScaleStartQuote((0, utils_1.toNative)(5000, 6).toNumber())
        .build());
    await client.tokenEdit(group, group.getFirstBankByMint(new web3_js_1.PublicKey(MAINNET_MINTS.get('BONK'))).mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams)
        .depositWeightScaleStartQuote((0, utils_1.toNative)(100000, 6).toNumber())
        .borrowWeightScaleStartQuote((0, utils_1.toNative)(100000, 6).toNumber())
        .build());
}
async function makePerpMarketReduceOnly() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = result[2];
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    const perpMarket = group.getPerpMarketByName('MNGO-PERP-OLD');
    await client.perpEditMarket(group, perpMarket.perpMarketIndex, (0, builder_1.Builder)(clientIxParamBuilder_1.NullPerpEditParams).reduceOnly(true).build());
}
async function createAndPopulateAlt() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const creator = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(MB_PAYER_KEYPAIR, 'utf-8'))));
    console.log(`Creator ${creator.publicKey.toBase58()}`);
    const group = await client.getGroupForCreator(creator.publicKey, GROUP_NUM);
    const connection = client.program.provider.connection;
    // Create ALT, and set to group at index 0
    if (group.addressLookupTables[0].equals(web3_js_1.PublicKey.default)) {
        try {
            console.log(`ALT: Creating`);
            const createIx = web3_js_1.AddressLookupTableProgram.createLookupTable({
                authority: admin.publicKey,
                payer: admin.publicKey,
                recentSlot: await connection.getSlot('finalized'),
            });
            const createTx = await (0, utils_1.buildVersionedTx)(client.program.provider, [createIx[0]]);
            let sig = await connection.sendTransaction(createTx);
            console.log(`...created ALT ${createIx[1]} https://explorer.solana.com/tx/${sig}`);
            console.log(`ALT: set at index 0 for group...`);
            sig = (await client.altSet(group, createIx[1], 0)).signature;
            console.log(`...https://explorer.solana.com/tx/${sig}`);
        }
        catch (error) {
            console.log(error);
        }
    }
    // Extend using mango v4 relevant pub keys
    try {
        const bankAddresses = Array.from(group.banksMapByMint.values())
            .flat()
            .map((bank) => [bank.publicKey, bank.oracle, bank.vault])
            .flat()
            .concat(Array.from(group.banksMapByMint.values())
            .flat()
            .map((mintInfo) => mintInfo.publicKey));
        const serum3MarketAddresses = Array.from(group.serum3MarketsMapByExternal.values())
            .flat()
            .map((serum3Market) => serum3Market.publicKey);
        const serum3ExternalMarketAddresses = Array.from(group.serum3ExternalMarketsMap.values())
            .flat()
            .map((serum3ExternalMarket) => [
            serum3ExternalMarket.publicKey,
            serum3ExternalMarket.bidsAddress,
            serum3ExternalMarket.asksAddress,
        ])
            .flat();
        const perpMarketAddresses = Array.from(group.perpMarketsMapByMarketIndex.values())
            .flat()
            .map((perpMarket) => [
            perpMarket.publicKey,
            perpMarket.oracle,
            perpMarket.bids,
            perpMarket.asks,
            perpMarket.eventQueue,
        ])
            .flat();
        // eslint-disable-next-line no-inner-declarations
        async function extendTable(addresses) {
            await group.reloadAll(client);
            const alt = await client.program.provider.connection.getAddressLookupTable(group.addressLookupTables[0]);
            addresses = addresses.filter((newAddress) => alt.value?.state.addresses &&
                alt.value?.state.addresses.findIndex((addressInALt) => addressInALt.equals(newAddress)) === -1);
            if (addresses.length === 0) {
                return;
            }
            const extendIx = web3_js_1.AddressLookupTableProgram.extendLookupTable({
                lookupTable: group.addressLookupTables[0],
                payer: admin.publicKey,
                authority: admin.publicKey,
                addresses,
            });
            const extendTx = await (0, utils_1.buildVersionedTx)(client.program.provider, [extendIx]);
            const sig = await client.program.provider.connection.sendTransaction(extendTx);
            console.log(`https://explorer.solana.com/tx/${sig}`);
        }
        console.log(`ALT: extending using mango v4 relevant public keys`);
        await extendTable(bankAddresses);
        await extendTable([constants_1.OPENBOOK_PROGRAM_ID['mainnet-beta']]);
        await extendTable(serum3MarketAddresses);
        await extendTable(serum3ExternalMarketAddresses);
        // TODO: dont extend for perps atm
        await extendTable(perpMarketAddresses);
        // Well known addresses
        await extendTable([
            web3_js_1.SystemProgram.programId,
            web3_js_1.SYSVAR_RENT_PUBKEY,
            spl_1.TOKEN_PROGRAM_ID,
            spl_1.ASSOCIATED_TOKEN_PROGRAM_ID,
            spl_1.NATIVE_MINT,
            web3_js_1.SYSVAR_INSTRUCTIONS_PUBKEY,
            web3_js_1.ComputeBudgetProgram.programId,
        ]);
    }
    catch (error) {
        console.log(error);
    }
}
async function main() {
    try {
        // await createGroup();
        // await changeAdmin();
        // await setDepositLimit();
    }
    catch (error) {
        console.log(error);
    }
    try {
        // await registerTokens();
        // await changeTokenOracle();
        // await makeTokenReduceonly();
        // await changeMaxStalenessSlots();
        // await changeStartQuote();
    }
    catch (error) {
        console.log(error);
    }
    try {
        // await registerSerum3Markets();
    }
    catch (error) {
        console.log(error);
    }
    try {
        // await registerPerpMarkets();
        // await makePerpMarketReduceOnly();
    }
    catch (error) {
        console.log(error);
    }
    try {
        // await createUser(MB_USER_KEYPAIR!);
        // depositForUser(MB_USER_KEYPAIR!);
    }
    catch (error) {
        console.log(error);
    }
    try {
        createAndPopulateAlt();
    }
    catch (error) {
        console.log(error);
    }
}
try {
    main();
}
catch (error) {
    console.log(error);
}
////////////////////////////////////////////////////////////
/// UNUSED /////////////////////////////////////////////////
////////////////////////////////////////////////////////////
async function expandMangoAccount(userKeypair) {
    const result = await buildUserClient(userKeypair);
    const client = result[0];
    const group = result[1];
    const user = result[2];
    const mangoAccounts = await client.getMangoAccountsForOwner(group, user.publicKey);
    if (!mangoAccounts) {
        throw new Error(`MangoAccounts not found for user ${user.publicKey}`);
    }
    for (const mangoAccount of mangoAccounts) {
        console.log(`...expanding MangoAccount ${mangoAccount.publicKey.toBase58()}`);
        await client.expandMangoAccount(group, mangoAccount, 8, 8, 8, 8);
    }
}
async function placeSerum3TradeAndCancelIt(userKeypair) {
    const result = await buildUserClient(userKeypair);
    const client = result[0];
    const group = result[1];
    const user = result[2];
    const mangoAccounts = await client.getMangoAccountsForOwner(group, user.publicKey);
    if (!mangoAccounts) {
        throw new Error(`MangoAccounts not found for user ${user.publicKey}`);
    }
    for (const mangoAccount of mangoAccounts) {
        console.log(`...found MangoAccount ${mangoAccount.publicKey.toBase58()}`);
        console.log(`...placing serum3 order`);
        await client.serum3PlaceOrder(group, mangoAccount, new web3_js_1.PublicKey(MAINNET_SERUM3_MARKETS.get('SOL/USDC')), serum3_1.Serum3Side.bid, 1, 1, serum3_1.Serum3SelfTradeBehavior.decrementTake, serum3_1.Serum3OrderType.limit, Date.now(), 10);
        console.log(`...current own orders on OB`);
        let orders = await mangoAccount.loadSerum3OpenOrdersForMarket(client, group, new web3_js_1.PublicKey(MAINNET_SERUM3_MARKETS.get('SOL/USDC')));
        for (const order of orders) {
            console.log(order);
        }
        console.log(`...cancelling serum3 orders`);
        await client.serum3CancelAllOrders(group, mangoAccount, new web3_js_1.PublicKey(MAINNET_SERUM3_MARKETS.get('SOL/USDC')), 10);
        console.log(`...current own orders on OB`);
        orders = await mangoAccount.loadSerum3OpenOrdersForMarket(client, group, new web3_js_1.PublicKey(MAINNET_SERUM3_MARKETS.get('SOL/USDC')));
        for (const order of orders) {
            console.log(order);
        }
    }
}
async function deregisterSerum3Markets() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    // change xxx/xxx to market of choice
    const serum3Market = group.getSerum3MarketByName('XXX/XXX');
    const sig = await client.serum3deregisterMarket(group, serum3Market.serumMarketExternal);
    console.log(`...deregistered serum market ${serum3Market.name}, sig https://explorer.solana.com/tx/${sig}`);
}
async function deregisterTokens() {
    const result = await buildAdminClient();
    const client = result[0];
    const admin = result[1];
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    // change -1 to tokenIndex of choice
    const bank = group.getFirstBankByTokenIndex(-1);
    const sig = await client.tokenDeregister(group, bank.mint);
    console.log(`...removed token ${bank.name}, sig https://explorer.solana.com/tx/${sig}`);
}
