"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const listingTools_1 = require("@blockworks-foundation/mango-v4-settings/lib/helpers/listingTools");
const anchor_1 = require("@coral-xyz/anchor");
const spl_governance_1 = require("@solana/spl-governance");
const web3_js_1 = require("@solana/web3.js");
const solana_js_1 = require("@switchboard-xyz/solana.js");
const fs_1 = __importDefault(require("fs"));
const src_1 = require("../src");
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const constants_2 = require("./governanceInstructions/constants");
const createProposal_1 = require("./governanceInstructions/createProposal");
const voteStakeRegistryClient_1 = require("./governanceInstructions/voteStakeRegistryClient");
const { MB_CLUSTER_URL, PROPOSAL_TITLE, PROPOSAL_LINK, VSR_DELEGATE_KEYPAIR, VSR_DELEGATE_FROM_PK, DRY_RUN, } = process.env;
async function buildClient() {
    return await client_1.MangoClient.connectDefault(MB_CLUSTER_URL);
}
async function setupWallet() {
    const clientKeypair = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(VSR_DELEGATE_KEYPAIR, 'utf-8'))));
    const clientWallet = new anchor_1.Wallet(clientKeypair);
    return clientWallet;
}
async function setupVsr(connection, clientWallet) {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const provider = new anchor_1.AnchorProvider(connection, clientWallet, options);
    const vsrClient = await voteStakeRegistryClient_1.VsrClient.connect(provider, voteStakeRegistryClient_1.DEFAULT_VSR_ID);
    return vsrClient;
}
async function withdrawFeesToAdmin() {
    const [client, wallet] = await Promise.all([buildClient(), setupWallet()]);
    const vsrClient = await setupVsr(client.connection, wallet);
    const group = await client.getGroup(constants_1.MANGO_V4_MAIN_GROUP);
    const instructions = [];
    const sbProgram = await solana_js_1.SwitchboardProgram.load('mainnet-beta', client.connection);
    Array.from(group.banksMapByTokenIndex.values())
        .map((banks) => banks[0])
        .sort((a, b) => a.name.localeCompare(b.name))
        .forEach(async (bank) => {
        const tier = Object.values(listingTools_1.LISTING_PRESETS).find((x) => x.initLiabWeight.toFixed(1) === '1.8'
            ? x.initLiabWeight.toFixed(1) ===
                bank?.initLiabWeight.toNumber().toFixed(1) &&
                x.reduceOnly === bank.reduceOnly
            : x.initLiabWeight.toFixed(1) ===
                bank?.initLiabWeight.toNumber().toFixed(1));
        if (bank.oracleProvider == src_1.OracleProvider.Switchboard) {
            const aggregatorAccount = new solana_js_1.AggregatorAccount(sbProgram, bank.oracle);
            const data = await aggregatorAccount.loadData();
            console.log(`${bank.name} ${tier?.preset_name} ${data.minUpdateDelaySeconds}`);
        }
    });
    const tokenOwnerRecordPk = await (0, spl_governance_1.getTokenOwnerRecordAddress)(constants_2.MANGO_GOVERNANCE_PROGRAM, constants_2.MANGO_REALM_PK, constants_2.MANGO_MINT, new web3_js_1.PublicKey(VSR_DELEGATE_FROM_PK));
    const [tokenOwnerRecord, proposals] = await Promise.all([
        (0, spl_governance_1.getTokenOwnerRecord)(client.connection, tokenOwnerRecordPk),
        (0, spl_governance_1.getAllProposals)(client.connection, constants_2.MANGO_GOVERNANCE_PROGRAM, constants_2.MANGO_REALM_PK),
    ]);
    const walletSigner = wallet;
    if (!DRY_RUN) {
        const proposalAddress = await (0, createProposal_1.createProposal)(client.connection, walletSigner, constants_2.MANGO_DAO_WALLET_GOVERNANCE, tokenOwnerRecord, PROPOSAL_TITLE ? PROPOSAL_TITLE : 'Withdraw token fees in mango-v4', PROPOSAL_LINK ?? '', Object.values(proposals).length, instructions, vsrClient, false);
        console.log(proposalAddress.toBase58());
    }
}
async function main() {
    try {
        await withdrawFeesToAdmin();
    }
    catch (error) {
        console.log(error);
    }
}
try {
    main();
}
catch (error) {
    console.log(error);
}
