"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const CLUSTER = process.env.CLUSTER_OVERRIDE || 'mainnet-beta';
const CLUSTER_URL = process.env.CLUSTER_URL_OVERRIDE || process.env.MB_CLUSTER_URL;
const USER_KEYPAIR = process.env.USER_KEYPAIR_OVERRIDE || process.env.MB_PAYER_KEYPAIR;
const MANGO_ACCOUNT_PK = process.env.MANGO_ACCOUNT_PK;
const TOKEN_INDEX = Number(process.env.TOKEN_INDEX);
async function tokenDeposit() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(process.env.KEYPAIR || fs_1.default.readFileSync(USER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'get-program-accounts',
        prioritizationFee: 80000,
    });
    const liqor = await client.getMangoAccount(new web3_js_1.PublicKey(MANGO_ACCOUNT_PK));
    const group = await client.getGroup(liqor.group);
    const forceCloseTokenBank = group.getFirstBankByTokenIndex(TOKEN_INDEX);
    const mangoAccountsWithBorrows = (await client.getAllMangoAccounts(group)).filter((a) => a.getTokenBalanceUi(forceCloseTokenBank) < 0);
    for (const liqee of mangoAccountsWithBorrows) {
        console.log(`liqee ${liqee.publicKey}, ${liqee.getTokenBalanceUi(forceCloseTokenBank)}`);
        const sig = await client.tokenDeposit(group, liqee, forceCloseTokenBank.mint, -liqee.getTokenBalanceUi(forceCloseTokenBank));
        console.log(` - tokendeposit, sig https://explorer.solana.com/tx/${sig.signature}?cluster=${CLUSTER == 'devnet' ? 'devnet' : ''}`);
    }
}
tokenDeposit();
